@@ -1,4 +1,74 @@
 # -*- coding: utf-8 -*-
+"""Multi-level molecule (monomer)
+
+    The molecule is defined by the vector of energies of its states
+    and by the transition dipole moments between allowed transitions.
+
+    >>> m = Molecule([0.0, 1.0], "Alphonso")
+    >>> print("Hi, my name is", m.get_name()+"!")
+    Hi, my name is Alphonso!
+
+
+    Information about the molecule can be obtained simply by printing it
+    
+    >>> print(m)
+    <BLANKLINE>
+    quantarhei.Molecule object
+    ==========================
+       name = Alphonso  
+       position = None
+       number of electronic states = 2
+       # State properties
+       State nr: 0 (ground state)
+          electronic energy = 0.0 1/fs
+          number of vibrational modes = 0
+    <BLANKLINE>
+       State nr: 1
+          electronic energy = 1.0 1/fs
+          transition 0 -> 1 
+          transition dipole moment = [0.0, 0.0, 0.0]
+          number of vibrational modes = 0
+    <BLANKLINE>
+
+    
+    >>> import quantarhei as qr
+    >>> mol = qr.TestMolecule("two-levels-1-mode")
+    >>> print(mol)
+    <BLANKLINE>
+    quantarhei.Molecule object
+    ==========================
+       name = two-levels-1-mode  
+       position = None
+       number of electronic states = 2
+       # State properties
+       State nr: 0 (ground state)
+          electronic energy = 0.0 1/fs
+          number of vibrational modes = 1
+          # Mode properties
+          mode no. = 0 
+             frequency = 1.0 1/fs
+             shift = 0.0
+             nmax = 2
+       State nr: 1
+          electronic energy = 1.0 1/fs
+          transition 0 -> 1 
+          transition dipole moment = [0.0, 0.0, 0.0]
+          number of vibrational modes = 1
+          # Mode properties
+          mode no. = 0 
+             frequency = 1.0 1/fs
+             shift = 0.0
+             nmax = 2
+    
+
+    Class Details
+    -------------
+    
+    
+
+"""
+
+
 import numpy
 
 from ..utils import array_property
@@ -27,40 +97,24 @@ from ..qm import ReducedDensityMatrix
 
 from ..core.saveable import Saveable
 
+import quantarhei as qr
+
  
     
 class Molecule(UnitsManaged, Saveable):
     """Multi-level molecule (monomer)
 
-    The molecule is defined by the vector of energies of its states
-    and by the transition dipole moments between allowed transitions.
-    
+
     Parameters
     ----------
     
     name : str
         Monomer descriptor; a string identifying the monomer
+
     elenergies : list of real numbers
         List of electronic energies, one per state. It includes ground state
         energy. It wise to chose the ground state energy as zero. 
 
-    Properties
-    ----------
-    
-    position : real array
-       3D vector holding a position of the molecule in space
-       
-    elenergies : real array
-       vector of electronic energies
-       
-    dmoments : real array
-       matrix of transition dipole moments. One vector of the transition
-       dipole moment for each pair of electronic states
-
-        
-    Methods
-    -------
-    ... to be continued
     
     """
     
@@ -71,7 +125,7 @@ class Molecule(UnitsManaged, Saveable):
     # transition dipole moments
     dmoments = array_property('dmoments')    
     
-    # number of electronic states
+    # number of electronic statesarray_property
     nel      = Integer('nel')
     # number of allowed transitions
     nal      = Integer('nal')
@@ -102,18 +156,24 @@ class Molecule(UnitsManaged, Saveable):
         
         
         
-        # FIXME: check the order of energies (increasing order has to be enforced)        
-        # no vibrational modes is a defauls
+        # FIXME: check the order of energies (increasing order has 
+        # to be enforced) no vibrational modes is a defauls
         self.nmod = 0
         self.modes = []  
         
         # allowed transitions are now only between the ground state and
-        # the rest of the excited states
+        # the rest of the excited states are dark
         self.allowed_transitions = []
          
-        
+        # transition dipole moments
         self.dmoments = numpy.zeros((self.nel,self.nel,3)) 
         
+        # matrix of the transition widths
+        self.widths = None
+        
+        # matrix of dephasing rates
+        self.dephs = None
+        
         self._has_nat_lifetime = [False]*self.nel
         self._nat_lifetime = numpy.zeros(self.nel)
         self._nat_linewidth = numpy.zeros(self.nel)        
@@ -144,40 +204,34 @@ class Molecule(UnitsManaged, Saveable):
         self.model = None
         self.data = None
         self._data_type = None
-   
-    
-#    def save_as(self, root, name):
-#        rtg = self._create_root_group(root, name)
-#        rtg.attrs.create("name", numpy.string_(self.name))
-#        rtg.attrs.create("nel",self.nel)
-#        rtg.create_dataset("elenergies", data=self.elenergies)
-#
-#    def load_as(self, root, name):
-#        rtg = root[name]
-#        self.name = rtg.attrs["name"].decode("utf-8")
-#        self.nel = rtg.attrs["nel"]   
-#        self.elenergies = numpy.array(rtg["elenergies"])
 
-    # FIXME: attribute manager in UnitsManager class complicates saving
-    def _before_save(self):
-        
-        self.manager = None
-        
-    def _after_save(self):
-        
-        self.manager = Manager()
-        
-
-    def _after_load(self):
-        """Set the manager as in UnitsManaged constructor
-        
-        """
-        self.manager = Manager()
         
     def get_name(self):
+        """Returns the name of the molecule
+        
+        Examples
+        --------
+        
+        >>> m = Molecule([0.0, 1.0], name="Jane")
+        >>> m.get_name()
+        'Jane'
+        
+        """
         return self.name
+
         
     def set_name(self, name):
+        """Sets the name of the Molecule object
+        
+        Examples
+        --------
+        
+        >>> m = Molecule([0.0, 1.0])
+        >>> m.set_name("Jane")
+        >>> print(m.get_name())
+        Jane
+        
+        """
         self.name = name
         
         
@@ -194,7 +248,7 @@ class Molecule(UnitsManaged, Saveable):
             A tuple describing a transition in the molecule, e.g. (0,1) is
             a transition from the ground state to the first excited state.
             
-        correlation_matrix : cu.oqs.correlationfunctions.CorrelationFunctionMatrix
+        correlation_matrix : CorrelationFunctionMatrix
             An instance of CorrelationFunctionMatrix
             
         position : int
@@ -277,10 +331,7 @@ class Molecule(UnitsManaged, Saveable):
                                            
             self._has_egcf[self.triangle.locate(transition[0],
                                                 transition[1])] = False
-                                                
-
         
-                                                
     #@deprecated
     def set_egcf(self, transition, egcf):
         self.set_transition_environment(transition, egcf)
@@ -304,7 +355,8 @@ class Molecule(UnitsManaged, Saveable):
                 return self.egcf_matrix.cfunc[iof]
 
         raise Exception("No environment set for the transition")   
-        
+
+   
     #@deprecated
     def get_egcf(self, transition): 
         
@@ -315,9 +367,29 @@ class Molecule(UnitsManaged, Saveable):
         return self.get_transition_environment(transition).data         
             
     
-    def add_Mode(self,mod):
-        """Adds a vibrational mode to the monomer"""
-        if isinstance(mod,Mode):
+    def add_Mode(self, mod):
+        """Adds a vibrational mode to the monomer
+        
+        Parameters
+        ----------
+        
+        mod : quantarhei.Mode
+            Intramolecular vibrational mode
+            
+            
+        Examples
+        --------
+        
+        >>> mode = qr.Mode()
+        >>> mol = Molecule([0.0, 2.0])
+        >>> print(mol.get_number_of_modes())
+        0
+        >>> mol.add_Mode(mode)
+        >>> print(mol.get_number_of_modes())
+        1
+        
+        """
+        if isinstance(mod, Mode):
             mod.set_Molecule(self)
             self.modes.append(mod)
             self.nmod += 1
@@ -327,38 +399,228 @@ class Molecule(UnitsManaged, Saveable):
     #
     # Some getters and setters 
     #
-    def get_Mode(self,N):
+    def get_Mode(self, N):
+        """Returns the Nth mode of the Molecule object
+        
+        
+        Parameters
+        ----------
+        
+        N : int
+            Index of the mode to be returned
+            
+            
+        Examples
+        --------
+        
+        >>> import quantarhei as qr
+        >>> mol = qr.TestMolecule("two-levels-1-mode")
+        >>> mod = mol.get_Mode(1)
+        Traceback (most recent call last):
+            ...
+        IndexError: list index out of range
+        
+        >>> mod = mol.get_Mode(0)
+        >>> mod.get_energy(1)
+        1.0
+        
+        """
+        
+        return self.modes[N]
+
+   
+    def get_number_of_modes(self):
+        """Retruns the number of modes in this molecule
+        
+        
+        Examples
+        --------
+        
+
+        >>> m = Molecule([0.0, 1.0])
+        >>> m.get_number_of_modes()
+        0
+
+        >>> import quantarhei as qr
+        >>> m = qr.TestMolecule("two-levels-1-mode")
+        >>> m.get_number_of_modes()
+        1
+        
+        """
+        return len(self.modes)
+
+        
+    def get_dipole(self, N, M):
         try:
-            return self.modes[N]
+            return self.dmoments[N, M, :]
         except:
             raise Exception()
+
             
-    def get_dipole(self,N,M):
-        try:
-            return self.dmoments[N,M,:]
-        except:
-            raise Exception()
-            
-    def set_dipole(self,N,M,vec):
+    def set_dipole(self, N, M, vec):
         if N == M:
             raise Exception("M must not be equal to N")
         try:
-            self.dmoments[N,M,:] = vec
-            self.dmoments[M,N,:] = numpy.conj(vec)
+            self.dmoments[N, M, :] = vec
+            self.dmoments[M, N, :] = numpy.conj(vec)
         except:
-            raise Exception()        
+            raise Exception()  
+
+            
+    def set_transition_width(self, transition, width):
+        """Sets the width of a given transition
+        
+        
+        Parameters
+        ----------
+        
+        transition : {tuple, list}
+            Quantum numbers of the states between which the transition occurs
+            
+        width : float
+            The full width at half maximum (FWHM) of a Gaussian lineshape,
+            or half width at half maximum (HWHM) of a Lorentzian lineshape
+            
+            
+        """
+        cwidth = qr.Manager().convert_energy_2_internal_u(width)
+        if self.widths is None:
+            N = self.elenergies.shape[0]
+            self.widths = numpy.zeros((N, N), dtype=qr.REAL)
+        self.widths[transition[0], transition[1]] = cwidth
+        self.widths[transition[1], transition[0]] = cwidth
+
+
+    def get_transition_width(self, transition):
+        """Returns the transition width
+        
+        
+        Returns the full width at half maximum (FWHM) of a Gaussian lineshape,
+        or half width at half maximum (HWHM) of a Lorentzian lineshape
+        
+        Parameters
+        ----------
+        
+        transition : {tuple, list}
+            Quantum numbers of the states between which the transition occurs
+            
+        """
+        
+        if self.widths is None:
+            return 0.0
+        
+        return self.widths[transition[0], transition[1]]
+
+    
+    def set_transition_dephasing(self, transition, deph):
+        """Sets the dephasing rate of a given transition
+        
+        
+        Parameters
+        ----------
+        
+        transition : {tuple, list}
+            Quantum numbers of the states between which the transition occurs
+            
+        deph : float
+            Dephasing rate of the transition
+            
+            
+        """        
+        if self.dephs is None:
+            N = self.elenergies.shape[0]
+            self.dephs = numpy.zeros((N, N), dtype=qr.REAL)
+        self.dephs[transition[0], transition[1]] = deph
+        self.dephs[transition[1], transition[0]] = deph
+
+
+    def get_transition_dephasing(self, transition):
+        """Returns the dephasing rate of a given transition
+        
+        
+        Parameters
+        ----------
+        
+        transition : {tuple, list}
+            Quantum numbers of the states between which the transition occurs
+            
+        """
+        
+        if self.dephs is None:
+            return 0.0
+        
+        return self.dephs[transition[0], transition[1]]
+
 
     def get_energy(self, N):
+        """Returns energy of the Nth state of the molecule
+        
+        
+        Parameters
+        ----------
+        
+        N : int
+            Index of the state
+            
+            
+        Examples
+        --------
+        
+        >>> import quantarhei as qr
+        >>> mol = qr.TestMolecule("two-levels-1-mode")
+        >>> mol.get_energy(1)
+        1.0
+        
+        This methods reacts to the `energy_units` context manager
+        
+        >>> with qr.energy_units("1/cm"):
+        ...     print("{0:.8f}".format(mol.get_energy(1)))
+        5308.83745888
+        
+        """
         try:
             return self.convert_energy_2_current_u(self.elenergies[N])
         except:
             raise Exception()
-            
+
+      
     def set_energy(self, N, en):
+        """Sets the energy of the Nth state of the molecule
+        
+        
+        Parameters
+        ----------
+        
+        N : int
+            Index of the state
+            
+        en : float
+            Energy to be assigned to the Nth state.
+            
+            
+        Examples
+        --------
+        
+        >>> import quantarhei as qr
+        >>> mol = qr.TestMolecule("two-levels-1-mode")
+        >>> mol.set_energy(1, 1.5)
+        >>> mol.get_energy(1)
+        1.5
+
+        This method reacts to the `energy_units` context manager
+
+        >>> import quantarhei as qr
+        >>> mol = qr.TestMolecule("two-levels-1-mode")
+        >>> with qr.energy_units("1/cm"):
+        ...     mol.set_energy(1, 5308.8374588761453)
+        >>> mol.get_energy(1)
+        1.0
+
+        """
         self.elenergies[N] = self.convert_energy_2_internal_u(en)
         
             
-    def set_electronic_natural_lifetime(self,N,epsilon_r=1.0):
+    def set_electronic_natural_lifetime(self, N, epsilon_r=1.0):
         
         rate = 0.0
         eps = eps0_int*epsilon_r
@@ -384,14 +646,13 @@ class Molecule(UnitsManaged, Saveable):
             self._nat_linewidth[N] = 1.0/lftm
             self._saved_epsilon_r = epsilon_r
             
-            
         else:
             #raise Exception("Cannot calculate natural lifetime")
             self._has_nat_lifetime[N] = True
             self._nat_lifetime[N] = numpy.inf   
             
             
-    def get_electronic_natural_lifetime(self,N,epsilon_r=1.0):
+    def get_electronic_natural_lifetime(self, N, epsilon_r=1.0):
         """Returns natural lifetime of a given electronic state
         
         """            
@@ -428,8 +689,7 @@ class Molecule(UnitsManaged, Saveable):
             rdm = ReducedDensityMatrix(data=dat)
                 
         
-        return rdm
-        
+        return rdm        
         
         
     def get_temperature(self):
@@ -458,7 +718,6 @@ class Molecule(UnitsManaged, Saveable):
             
             raise Exception("Molecular environment has"+
             "an inconsisten temperature")
-
         
         
     def check_temperature_consistent(self):
@@ -467,7 +726,8 @@ class Molecule(UnitsManaged, Saveable):
         """
         #FIXME: implement the check
         return True
-        
+
+     
     def set_adiabatic_coupling(self,state1,state2,coupl):
         """Sets adiabatic coupling between two states
         
@@ -483,6 +743,7 @@ class Molecule(UnitsManaged, Saveable):
         self.adiabatic_coupling[self.triangle.locate(state1,state2)] = cp
         self._has_adiabatic[self.triangle.locate(state1,state2)] = True
 
+
     def get_adiabatic_coupling(self,state1,state2):
         """Returns adiabatic coupling between two states
         
@@ -490,8 +751,6 @@ class Molecule(UnitsManaged, Saveable):
         """
         return self.adiabatic_coupling[self.triangle.locate(state1,state2)]
         
-        
-
          
     def get_electronic_natural_linewidth(self,N):
         """Returns natural linewidth of a given electronic state
@@ -508,6 +767,21 @@ class Molecule(UnitsManaged, Saveable):
         """Returns the Hamiltonian of the Molecule object
         
         
+        Examples
+        --------
+        
+        >>> import quantarhei as qr
+        >>> mol = qr.TestMolecule("two-levels-1-mode")
+        >>> H = mol.get_Hamiltonian()
+        >>> print(H.dim)
+        4
+        
+        >>> print(H.data)
+        [[ 0.  0.  0.  0.]
+         [ 0.  1.  0.  0.]
+         [ 0.  0.  1.  0.]
+         [ 0.  0.  0.  2.]]
+        
         """
         # list of vibrational Hamiltonians
         lham = [None]*self.nel  
@@ -582,7 +856,8 @@ class Molecule(UnitsManaged, Saveable):
                     if self._has_adiabatic[self.triangle.locate(i,j)]:
                         J = self.get_adiabatic_coupling(i,j)
                         hj = numpy.zeros((ldim[i],ldim[j]),dtype=numpy.float)
-                        # FIXME: this works only if the frequencies of the oscillators are the same
+                        # FIXME: this works only if the frequencies
+                        # of the oscillators are the same
                         for k in range(min([ldim[i],ldim[j]])):
                             hj[k,k] = J
                         ham[lb[i]:ub[i],lb[j]:ub[j]] = hj
@@ -629,8 +904,7 @@ class Molecule(UnitsManaged, Saveable):
         totdim = numpy.sum(ldim)
 
         return totdim, ldim        
-        
-        
+               
         
     def get_TransitionDipoleMoment(self):
 
@@ -696,7 +970,8 @@ class Molecule(UnitsManaged, Saveable):
                                               # the baths
                     else:
                         where[eg] = [(nob,nob)]
-                    # for each bath, save the state of the traprint(mols_c2)  nsition g -> j
+                    # for each bath, save the state of the traprint(mols_c2)
+                    # nsition g -> j
                     d[nob] = j
                         
                     nob += 1
@@ -742,8 +1017,7 @@ class Molecule(UnitsManaged, Saveable):
 #            for w in wr:
 #                char = d[w[0]]
 #                print("fce ",el, " is a bath no.", w[0]," of ",char)
-                
-            cfm.set_correlation_function(i+1,el,wr)
+            cfm.set_correlation_function(el,wr,i+1)
 
         #
         # System operators corresponding to the correlation functions
@@ -802,7 +1076,7 @@ class Molecule(UnitsManaged, Saveable):
         return SystemBathInteraction(sys_operators,cfm)
         
     
-    def set_mode_environment(self,mode=0,elstate=0,corfunc=None):
+    def set_mode_environment(self, mode=0, elstate=0, corfunc=None):
         """Sets mode environment 
         
         
@@ -913,7 +1187,160 @@ class Molecule(UnitsManaged, Saveable):
                            self.modes[m1].get_nmax(n))
             
         return out
+
+      
+    def liouville_pathways_1(self, eUt=None, ham=None, dtol=0.01, ptol=1.0e-3,
+                             etol=1.0e-6, verbose=0, lab=None):
+        """ Generator of the first order Liouville pathways 
         
+        
+        Generator of the pathways for an absorption spectrum
+        calculation.
+        
+        
+        
+        Parameters
+        ----------
+        
+            
+        eUt : EvolutionSuperOperator
+            Evolution superoperator representing the evolution of optical
+            coherence in the system 
+            
+            
+        dtol : float
+            Minimum acceptable strength of the transition from ground
+            to excited state, relative to the maximum dipole strength 
+            available in the system
+            
+        ptol : float
+            Minimum acceptable population of the ground state (e.g. states
+            not thermally populated are excluded)
+
+        lab : LaboratorySetup
+            Object representing laboratory setup - number of pulses, 
+            polarization etc.
+            
+        Returns
+        -------
+        
+        lst : list
+            List of LiouvillePathway objects
+            
+            
+        """
+        if self._diagonalized:
+            if verbose > 0:
+                print("Diagonalizing aggregate")
+            self.diagonalize()
+            if verbose > 0:
+                print("..done")
+        
+        pop_tol = ptol
+        dip_tol = numpy.sqrt(self.D2_max)*dtol
+        evf_tol = etol
+                        
+        if eUt is None:
+            
+            # secular absorption spectrum calculation
+            eUt2_dat = None
+            sec = True
+        
+        else:
+            
+            raise Exception("Not implemented yet")
+
+        lst = []        
+
+        if sec:
+            generate_1orderP_sec(self, lst,
+                                 pop_tol, dip_tol, verbose)
+        else:
+            raise Exception("Not implemented yet")                                
+        
+        if lab is not None:
+            for l in lst:
+                l.orientational_averaging(lab)
+         
+        return lst   
+
+def generate_1orderP_sec(self, lst,
+                         pop_tol, dip_tol, verbose):
+    
+    ngs = (0) # self.get_electronic_groundstate()
+    nes = (1) #self.get_excitonic_band(band=1)
+    
+    if verbose > 0:
+        print("Liouville pathway of first order")
+        print("Population tolerance: ", pop_tol)
+        print("Dipole tolerance:     ", dip_tol)
+
+    k = 0
+    l = 0
+    for i1g in ngs:
+
+        if verbose > 0: 
+            print("Ground state: ", i1g, "of", len(ngs))
+        
+        # Only thermally allowed starting states are considered
+        if True: #self.rho0[i1g,i1g] > pop_tol:
+    
+            D2 = numpy.dot(self.dmoments[0,1,:], self.dmoments[0,1,:])
+            for i2e in nes:
+                
+                if D2 > dip_tol: #self.D2[i2e,i1g] > dip_tol:
+                
+                                
+                    l += 1
+                                
+
+                    #      Diagram P1
+                    #
+                    #                                     
+                    #      |g_i1> <g_i1|
+                    # <----|-----------|
+                    #      |e_i2> <g_i1|
+                    # ---->|-----------|
+                    #      |g_i1> <g_i1|
+
+                    try:
+                        if verbose > 5:
+                            print(" * Generating P1", i1g, i2e)
+
+                        lp = \
+                        diag.liouville_pathway("NR",
+                                               i1g,
+                                    aggregate=self,
+                                    order=1,pname="P1",
+                                    popt_band=1,
+                                    relax_order=1)
+                        
+                        # first transition lineshape
+                        width1 = \
+                            self.get_transition_width((i2e, i1g))
+                        deph1 = \
+                            self.get_transition_dephasing((i2e, 
+                                               i1g))
+                        
+                        #      |g_i1> <g_i1|                                                           
+                        lp.add_transition((i2e,i1g),+1,
+                                          interval=1, 
+                                          width=width1, 
+                                          deph=deph1)
+                        #      |e_i2> <g_i1|
+                        lp.add_transition((i1g,i2e),+1,
+                                          interval=1, 
+                                          width=width1, 
+                                          deph=deph1)
+                        #      |g_i1> <g_i1|
+
+                    except:
+                        
+                        break
+                    
+                    lp.build()
+                    lst.append(lp)
+                    k += 1
 
 def PiMolecule(Molecule):
     
